/*
 *     Copyright: Eugene M. Hutorny (c) 2009, eugene@hutorny.in.ua
 *     License:   Licensed to public under The GNU Lesser General Public License (LGPLv3) 
 *                http://www.opensource.org/licenses/lgpl-3.0.html
 */
package ua.in.hutorny.otj.exer1;

import ua.in.hutorny.otj.exer1.accounting.Account;
import ua.in.hutorny.otj.exer1.bank.Bank;
import ua.in.hutorny.otj.exer1.bank.Cashier;
import ua.in.hutorny.otj.exer1.bank.WireTransfer;
import ua.in.hutorny.otj.exer1.exeptions.NotEnoughCash;
import ua.in.hutorny.otj.exer1.exeptions.NotEnoughFunds;
import ua.in.hutorny.otj.exer1.money.Currency;


public class TomAndMaryPlan {
	
	/**
	 * Entities that exists regardless of this plan
	 */
	Bank spicer  = new Bank("JVM-Spicer,MN ", Currency.USD);
	Bank willmar = new Bank("OTJ-Willmar,MN", Currency.EUR);
	Bank bahama  = new Bank("Java-Bahamas  ", Currency.BSD);
	
	Person tom  = new Person("Tom ");
	Person mary = new Person("Mary");
	/**
	 * List of accounts to keep track of during this plan
	 */
	Account[] accounts = new Account[]{};
	
	/**
	 * Plan execution routine 
	 */
	void run() throws NotEnoughFunds, NotEnoughCash {

		Account credit = spicer.openCreditAccount(1000000, Currency.USD);				
		accounts = new Account[]{credit};										_(1);
		Cashier cashier = spicer.cashier(credit); 						
		cashier.giveCashTo(tom, 970000);										_(2);
		
		mary.takes(	tom.gives(965000, Currency.USD) );							_(3);
		
		Account deposit = willmar.openAccount(mary.gives(960000,Currency.USD));
		accounts = new Account[]{credit, deposit}; 								_(4);

		System.out.println("\n" + mary + " flies to Bahama");
		
		Account bahamamama = bahama.openAccount(mary.gives(1000,Currency.USD));
		accounts = new Account[]{credit, deposit, bahamamama};					_(5);

		WireTransfer wire = willmar.wireOperator(deposit);
		wire.wireTo(bahamamama, 950000); 										_(6);

		cashier = bahama.cashier(bahamamama);
		cashier.giveCashTo(mary, 923000);										_(7);		
		
		System.out.println("\n" + tom + " flies to Bahama and meets " + mary);
		tom.takes( mary.gives(461000, Currency.USD) );							_(8);		
	}
	/**
	 * A reporting function to visualize the money flows 
	 */
	public void _(int step) {		
		System.out.println("\n" + step + " -- ");
		System.out.println(tom);
		System.out.println(mary);
		System.out.println();
		System.out.println(spicer.report());
		System.out.println(willmar.report());
		System.out.println(bahama.report());
		for(Account b : accounts) {
			System.out.println();
			System.out.print(b.toString());
		}
		System.out.println();
	}
}
