/*
 *     Copyright: Eugene M. Hutorny (c) 2009, eugene@hutorny.in.ua
 *     License:   Licensed to public under The GNU Lesser General Public License (LGPLv3) 
 *                http://www.opensource.org/licenses/lgpl-3.0.html
 */
package ua.in.hutorny.otj.exer1.accounting;
import java.math.BigDecimal;
import ua.in.hutorny.otj.exer1.exeptions.NotEnoughFunds;
import ua.in.hutorny.otj.exer1.money.Currency;

public abstract class Account {

	private BigDecimal amount;
	final private Currency currency;

	protected Account(BigDecimal deposit, Currency currency) {
		super();
		this.currency = currency;
		amount = deposit;
	}
	
	protected Account(Currency currency) {
		super();
		this.currency = currency;
		amount = new BigDecimal(0);
	}

	protected void debit(BigDecimal amount) {
		this.amount = this.amount.add(amount); 
	}

	protected void credit(BigDecimal amount) throws NotEnoughFunds {
		BigDecimal value = this.amount.subtract(amount);
		if( value.doubleValue() < 0 )
			throw new NotEnoughFunds();
		this.amount = value;
	}
	
	public Currency currency() { return currency; }
	
	@Override
	public String toString() {
		return getClass().getSimpleName() +"(" + currency.name() + ":" + String.format("% 9.2f", amount) + ")";
	}
}

