/*
 *     Copyright: Eugene M. Hutorny (c) 2009, eugene@hutorny.in.ua
 *     License:   Licensed to public under The GNU Lesser General Public License (LGPLv3) 
 *                http://www.opensource.org/licenses/lgpl-3.0.html
 */
package ua.in.hutorny.otj.exer1.bank;

import java.math.BigDecimal;

import ua.in.hutorny.otj.exer1.accounting.Account;
import ua.in.hutorny.otj.exer1.accounting.WithdrawCollaboration;
import ua.in.hutorny.otj.exer1.bank.Bank.BankAccount;
import ua.in.hutorny.otj.exer1.exeptions.NotEnoughFunds;
import ua.in.hutorny.otj.exer1.money.Currency;

/**
 * Wire Transfer Collaboration is a Withdraw Collaboration with
 * a bank as operator and currency converter, and two banking accounts (debit,credit)
 *
 */
team class WireTransferCollaboration extends WithdrawCollaboration implements WireTransfer {

	public WireTransferCollaboration(Bank bank, BankAccount creditAccount,double interest) {
		super();
		this.bank = bank;
		this.interest = interest;
		this.account = creditAccount;
	}

	/**
	 * Role operator is added to provide the collaboration with the operational account
	 * effective at the moment of transaction, which may be no the moment when 
	 * the collaboration was created  
	 */
	protected class Operator playedBy Bank {
		OperationalAccount operationalAccount() -> Account operationalAccount();		
	}
	
	@SuppressWarnings("bindingconventions")
	protected class DebitAccount playedBy Account {
		@SuppressWarnings("decapsulation")
		// how to add assertion on Account.this.currency() == currency ??  
		void debit(BigDecimal amount, Currency currency) -> void debit(BigDecimal amount);
	}	

	public void wireTo(Account debitAccount, int amount) throws NotEnoughFunds {
		if( debitAccount.currency() != account.currency() )
			throw new IllegalArgumentException("Debit account should have the same currency (" + account.currency() + ") as Credit account");
		withdraw(new BigDecimal(amount), account, debitAccount, bank, bank);
	}
		
	protected void withdraw(BigDecimal amount, BankAccount as CreditAccount creditAccount, Account as DebitAccount debitAccount,
			Bank as Operator operator, Bank as CurrencyConverter currencyConverter) throws NotEnoughFunds {
		super.withdraw(amount, creditAccount, debitAccount, operator.operationalAccount(), currencyConverter);
	}

	@Override
	protected BigDecimal calculateFee(BigDecimal amount) {
		return new BigDecimal(amount.doubleValue() * interest);
	}
	protected class CurrencyConverter playedBy Bank {
		BigDecimal convert(Currency from, Currency to, BigDecimal amount) -> 
			BigDecimal internalConversion(Currency from, BigDecimal amount)
			with { from -> from, amount -> amount, result <- result }
	}
		
	protected class CreditAccount playedBy Bank.BankAccount {
		@SuppressWarnings("decapsulation")
		credit -> credit;
		currency -> currency;
	}
	
	@SuppressWarnings("bindingconventions")
	protected class OperationalAccount playedBy Account {
		@SuppressWarnings("decapsulation")
		debit -> debit;
		currency -> currency;		
	}
	protected final Bank bank;
	protected final Bank.BankAccount account;
	protected final double interest;	
	
}
